# Generated by roxygen2: do not edit by hand

S3method(roclet_process,roclet_maestro)
S3method(roclet_process,roclet_maestroDays)
S3method(roclet_process,roclet_maestroFlags)
S3method(roclet_process,roclet_maestroFrequency)
S3method(roclet_process,roclet_maestroHours)
S3method(roclet_process,roclet_maestroInputs)
S3method(roclet_process,roclet_maestroLabel)
S3method(roclet_process,roclet_maestroLogLevel)
S3method(roclet_process,roclet_maestroMap)
S3method(roclet_process,roclet_maestroMonths)
S3method(roclet_process,roclet_maestroOutputs)
S3method(roclet_process,roclet_maestroPriority)
S3method(roclet_process,roclet_maestroRunIf)
S3method(roclet_process,roclet_maestroSkip)
S3method(roclet_process,roclet_maestroStartTime)
S3method(roclet_process,roclet_maestroTz)
S3method(roxy_tag_parse,roxy_tag_maestro)
S3method(roxy_tag_parse,roxy_tag_maestroDays)
S3method(roxy_tag_parse,roxy_tag_maestroFlags)
S3method(roxy_tag_parse,roxy_tag_maestroFrequency)
S3method(roxy_tag_parse,roxy_tag_maestroHours)
S3method(roxy_tag_parse,roxy_tag_maestroInputs)
S3method(roxy_tag_parse,roxy_tag_maestroLabel)
S3method(roxy_tag_parse,roxy_tag_maestroLogLevel)
S3method(roxy_tag_parse,roxy_tag_maestroMap)
S3method(roxy_tag_parse,roxy_tag_maestroMonths)
S3method(roxy_tag_parse,roxy_tag_maestroOutputs)
S3method(roxy_tag_parse,roxy_tag_maestroPriority)
S3method(roxy_tag_parse,roxy_tag_maestroRunIf)
S3method(roxy_tag_parse,roxy_tag_maestroSkip)
S3method(roxy_tag_parse,roxy_tag_maestroStartTime)
S3method(roxy_tag_parse,roxy_tag_maestroTz)
export(build_schedule)
export(create_maestro)
export(create_pipeline)
export(get_artifacts)
export(get_flags)
export(get_labels)
export(get_network)
export(get_run_sequence)
export(get_schedule)
export(get_slot_usage)
export(get_status)
export(invoke)
export(last_build_errors)
export(last_run_errors)
export(last_run_messages)
export(last_run_warnings)
export(run_schedule)
export(suggest_orch_frequency)
import(R6)
import(lifecycle)
import(logger)
import(tictoc)
import(utils)
importFrom(R.utils,countLines)
importFrom(roxygen2,roclet_output)
importFrom(roxygen2,roclet_process)
importFrom(roxygen2,roxy_tag_parse)
