#' Simple mtcars print function
#'
#' This is a function that runs every hour starting at
#' 2024-03-01 09:00:00
#'
#' @maestroFrequency 1 day
#' @maestroTz UTC
#'
#' @export
get_mtcars <- function() {
  mtcars
}

#' Multiply
#'
#' @maestroFrequency 3 months
#'
#' @export
multiply <- function(val, by) {
  val * by
}

#' Bad
#'
#' @maestroTz BLA
something_else <- function() {

}
