# maestroStartTime with HH:MM:SS runs on the expected time

    Code
      status$invoked
    Output
      [1] TRUE

# maestroPriority works as expected

    Code
      status$pipe_name
    Output
      [1] "q1" "p2"

