# Simple pipeline list, no errors

    Code
      pipeline_list
    Message
      
      -- Maestro Pipelines List with 1 pipeline 

---

    Code
      pipeline$get_status()[c("invoked", "success", "errors", "warnings", "messages")]
    Output
      # A tibble: 1 x 5
        invoked success errors warnings messages
        <lgl>   <lgl>    <int>    <int>    <int>
      1 TRUE    TRUE         0        0        0

