# get_slot_usage works as expected

    Code
      avail_hour
    Output
      # A tibble: 24 x 3
         slot  n_runs pipe_names               
         <chr>  <int> <chr>                    
       1 00:00      2 hourly1, hourly2         
       2 01:00      2 hourly1, hourly3         
       3 02:00      2 hourly1, hourly2         
       4 03:00      1 hourly1                  
       5 04:00      3 hourly1, hourly2, hourly3
       6 05:00      1 hourly1                  
       7 06:00      2 hourly1, hourly2         
       8 07:00      2 hourly1, hourly3         
       9 08:00      2 hourly1, hourly2         
      10 09:00      1 hourly1                  
      # i 14 more rows

---

    Code
      avail_day
    Output
      # A tibble: 31 x 3
         slot  n_runs pipe_names               
         <chr>  <int> <chr>                    
       1 01         3 hourly1, hourly2, hourly3
       2 02         3 hourly1, hourly2, hourly3
       3 03         3 hourly1, hourly2, hourly3
       4 04         3 hourly1, hourly2, hourly3
       5 05         3 hourly1, hourly2, hourly3
       6 06         3 hourly1, hourly2, hourly3
       7 07         3 hourly1, hourly2, hourly3
       8 08         3 hourly1, hourly2, hourly3
       9 09         3 hourly1, hourly2, hourly3
      10 10         3 hourly1, hourly2, hourly3
      # i 21 more rows

# get_slot_usage works with variety of frequencies

    Code
      avail_hour
    Output
      # A tibble: 24 x 3
         slot  n_runs pipe_names
         <chr>  <int> <chr>     
       1 00:00      1 hourly1   
       2 01:00      1 hourly1   
       3 02:00      1 hourly1   
       4 03:00      1 hourly1   
       5 04:00      1 hourly1   
       6 05:00      1 hourly1   
       7 06:00      1 hourly1   
       8 07:00      1 hourly1   
       9 08:00      1 hourly1   
      10 09:00      1 hourly1   
      # i 14 more rows

---

    Code
      avail_day
    Output
      # A tibble: 31 x 3
         slot  n_runs pipe_names              
         <chr>  <int> <chr>                   
       1 01         3 hourly1, daily1, weekly1
       2 02         3 hourly1, daily1, weekly1
       3 03         3 hourly1, daily1, weekly1
       4 04         3 hourly1, daily1, weekly1
       5 05         3 hourly1, daily1, weekly1
       6 06         3 hourly1, daily1, weekly1
       7 07         3 hourly1, daily1, weekly1
       8 08         3 hourly1, daily1, weekly1
       9 09         3 hourly1, daily1, weekly1
      10 10         3 hourly1, daily1, weekly1
      # i 21 more rows

---

    Code
      avail_week
    Output
      # A tibble: 7 x 3
        slot  n_runs pipe_names              
        <chr>  <int> <chr>                   
      1 Fri        2 hourly1, daily1         
      2 Mon        2 hourly1, daily1         
      3 Sat        2 hourly1, daily1         
      4 Sun        2 hourly1, daily1         
      5 Thu        3 hourly1, daily1, weekly1
      6 Tue        2 hourly1, daily1         
      7 Wed        2 hourly1, daily1         

