% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.run_sequence_min_days_out}
\alias{.run_sequence_min_days_out}
\title{Minimum days ahead to pre-compute run sequences on initialization
This small window is used at parse time; the full window is built lazily
when observability functions (get_run_sequence, get_slot_usage) are called.}
\usage{
.run_sequence_min_days_out(unit)
}
\arguments{
\item{unit}{character frequency unit}
}
\value{
integer
}
\description{
Minimum days ahead to pre-compute run sequences on initialization
This small window is used at parse time; the full window is built lazily
when observability functions (get_run_sequence, get_slot_usage) are called.
}
\keyword{internal}
